Attribute VB_Name = "D2D1"

' //
' // Module D2D1.bas - Helpes functions for Direct2D
' // By The trick 2018 (c)
' // Most part of functions is ported from D2D1helper.h (Microsoft Corporation (c))
' //

Option Explicit

Public Const FloatMax   As Single = 3.402823466E+38
Private Declare Function GetSysColor Lib "user32.dll" (ByVal nIndex As Long) As Long

' // Create ID2D1Factory object
Public Function CreateFactory( _
                Optional ByVal eType As D2D1_FACTORY_TYPE = D2D1_FACTORY_TYPE_SINGLE_THREADED, _
                Optional ByVal eDebugOptions As D2D1_DEBUG_LEVEL = -1) As ID2D1Factory
    Dim tOptions    As D2D1_FACTORY_OPTIONS
    Dim pOptions    As Long
    Dim cIID(1)     As Currency
    
    If eDebugOptions <> -1 Then
    
        tOptions.debugLevel = eDebugOptions
        pOptions = VarPtr(tOptions)
        
    End If
    
    ' // IID_ID2D1Factory
    cIID(0) = 506948672004902.9703@
    cIID(1) = 53149071617564.8146@
    
    Set CreateFactory = D2D1CreateFactory(eType, cIID(0), ByVal pOptions)
    
End Function

Public Function Point2F( _
                ByVal fX As Single, _
                ByVal fY As Single) As D2D1_POINT_2F
                
    Point2F.x = fX
    Point2F.y = fY
    
End Function

Public Function Point2U( _
                ByVal lX As Long, _
                ByVal lY As Long) As D2D1_POINT_2U
                
    Point2U.x = lX
    Point2U.y = lY
    
End Function

Public Function SizeU( _
                Optional ByVal lWidth As Long, _
                Optional ByVal lHeight As Long) As D2D1_SIZE_U
                
    SizeU.width = lWidth
    SizeU.height = lHeight
    
End Function

Public Function SizeF( _
                Optional ByVal fWidth As Single, _
                Optional ByVal fHeight As Single) As D2D1_SIZE_F
                
    SizeF.width = fWidth
    SizeF.height = fHeight
    
End Function

Public Function RectF( _
                Optional ByVal fLeft As Single, _
                Optional ByVal fTop As Single, _
                Optional ByVal fRight As Single, _
                Optional ByVal fBottom As Single) As D2D1_RECT_F
    
    With RectF
    
    .left = fLeft
    .right = fRight
    .top = fTop
    .bottom = fBottom
    
    End With
    
End Function

Public Function RectF_InfiniteRect() As D2D1_RECT_F
    
    With RectF_InfiniteRect
    
    .left = -FloatMax
    .right = FloatMax
    .top = -FloatMax
    .bottom = FloatMax
    
    End With
    
End Function

Public Function RectU( _
                Optional ByVal lLeft As Long, _
                Optional ByVal lTop As Long, _
                Optional ByVal lRight As Long, _
                Optional ByVal lBottom As Long) As D2D1_RECT_U
    
    With RectU
    
    .left = lLeft
    .right = lRight
    .top = lTop
    .bottom = lBottom
    
    End With
    
End Function

Public Function ArcSegment( _
                ByRef tPoint As D2D1_POINT_2F, _
                ByRef tSize As D2D1_SIZE_F, _
                ByVal fRotationAngle As Single, _
                ByVal eSweepDirection As D2D1_SWEEP_DIRECTION, _
                ByVal eArcSize As D2D1_ARC_SIZE) As D2D1_ARC_SEGMENT
    
    With ArcSegment
    
    .Point = tPoint
    .size = tSize
    .rotationAngle = fRotationAngle
    .sweepDirection = eSweepDirection
    .arcSize = eArcSize
    
    End With
    
End Function

Public Function BezierSegment( _
                ByRef tPoint1 As D2D1_POINT_2F, _
                ByRef tPoint2 As D2D1_POINT_2F, _
                ByRef tPoint3 As D2D1_POINT_2F) As D2D1_BEZIER_SEGMENT

    With BezierSegment
    
    .point1 = tPoint1
    .point2 = tPoint2
    .point3 = tPoint3
    
    End With
    
End Function

Public Function Ellipse( _
                ByRef tCenter As D2D1_POINT_2F, _
                ByVal fRadiusX As Single, _
                ByVal fRadiusY As Single) As D2D1_ELLIPSE
    
    With Ellipse
    
    .Point = tCenter
    .radiusX = fRadiusX
    .radiusY = fRadiusY
    
    End With
    
End Function

Public Function RoundedRect( _
                ByRef tRect As D2D1_RECT_F, _
                ByVal fRadiusX As Single, _
                ByVal fRadiusY As Single) As D2D1_ROUNDED_RECT
    
    With RoundedRect
    
    .rect = tRect
    .radiusX = fRadiusX
    .radiusY = fRadiusY
    
    End With
    
End Function
    
Public Function BrushProperties( _
                ByVal fOpacity As Single, _
                ByRef tTransform As D2D1_MATRIX_3X2_F) As D2D1_BRUSH_PROPERTIES
    
    With BrushProperties
    
    .opacity = fOpacity
    .Transform = tTransform
    
    End With
    
End Function

Public Function GradientStop( _
                ByVal fPosition As Single, _
                ByRef tColor As D2D1_COLOR_F) As D2D1_GRADIENT_STOP
    
    With GradientStop
    
    .Position = fPosition
    .color = tColor
    
    End With
    
End Function

Public Function QuadraticBezierSegment( _
                ByRef tPoint1 As D2D1_POINT_2F, _
                ByRef tPoint2 As D2D1_POINT_2F) As D2D1_QUADRATIC_BEZIER_SEGMENT
    
    With QuadraticBezierSegment
    
    .point1 = tPoint1
    .point2 = tPoint2
    
    End With
    
End Function

Public Function StrokeStyleProperties( _
                Optional ByVal eStartCap As D2D1_CAP_STYLE = D2D1_CAP_STYLE_FLAT, _
                Optional ByVal eEndCap As D2D1_CAP_STYLE = D2D1_CAP_STYLE_FLAT, _
                Optional ByVal eDashCap As D2D1_CAP_STYLE = D2D1_CAP_STYLE_FLAT, _
                Optional ByVal eLineJoin As D2D1_LINE_JOIN = D2D1_LINE_JOIN_MITER, _
                Optional ByVal fMiterLimit As Single = 10!, _
                Optional ByVal eDashStyle As D2D1_DASH_STYLE = D2D1_DASH_STYLE_SOLID, _
                Optional ByVal fDashOffset As Single) As D2D1_STROKE_STYLE_PROPERTIES
                
    With StrokeStyleProperties
    
    .startCap = eStartCap
    .endCap = eEndCap
    .dashCap = eDashCap
    .lineJoin = eLineJoin
    .miterLimit = fMiterLimit
    .DashStyle = eDashStyle
    .dashOffset = fDashOffset
    
    End With
    
End Function

Public Function BitmapBrushProperties( _
                Optional ByVal eExtendModeX As D2D1_EXTEND_MODE = D2D1_EXTEND_MODE_CLAMP, _
                Optional ByVal eExtendModeY As D2D1_EXTEND_MODE = D2D1_EXTEND_MODE_CLAMP, _
                Optional ByVal eInterpolationMode As D2D1_BITMAP_INTERPOLATION_MODE = _
                D2D1_BITMAP_INTERPOLATION_MODE_LINEAR) As D2D1_BITMAP_BRUSH_PROPERTIES
                
    With BitmapBrushProperties
    
    .extendModeX = eExtendModeX
    .extendModeY = eExtendModeY
    .interpolationMode = eInterpolationMode
    
    End With
    
End Function

Public Function LinearGradientBrushProperties( _
                ByRef tStartPoint As D2D1_POINT_2F, _
                ByRef tEndPoint As D2D1_POINT_2F) As D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES
                
    With LinearGradientBrushProperties
    
    .startPoint = tStartPoint
    .endPoint = tEndPoint
    
    End With
    
End Function

Public Function RadialGradientBrushProperties( _
                ByRef tCenter As D2D1_POINT_2F, _
                ByRef tGradientOriginOffset As D2D1_POINT_2F, _
                ByVal fRadiusX As Single, _
                ByVal fRadiusY As Single) As D2D1_RADIAL_GRADIENT_BRUSH_PROPERTIES
                
    With RadialGradientBrushProperties
    
    .center = tCenter
    .gradientOriginOffset = tGradientOriginOffset
    .radiusX = fRadiusX
    .radiusY = fRadiusY
    
    End With
    
End Function

Public Function pixelFormat( _
                Optional ByVal eDxgiFormat As DXGI_FORMAT = DXGI_FORMAT_UNKNOWN, _
                Optional ByVal eAlphaMode As D2D1_ALPHA_MODE = D2D1_ALPHA_MODE_UNKNOWN) As D2D1_PIXEL_FORMAT
                
    pixelFormat.Format = eDxgiFormat
    pixelFormat.alphaMode = eAlphaMode
    
End Function

Public Function BitmapProperties( _
                ByRef tPixelFormat As D2D1_PIXEL_FORMAT, _
                ByVal fDpiX As Single, _
                ByVal fDpiY As Single) As D2D1_BITMAP_PROPERTIES
                
    With BitmapProperties
    
    .pixelFormat = tPixelFormat
    .DpiX = fDpiX
    .DpiY = fDpiY
    
    End With
    
End Function

Public Function RenderTargetProperties( _
                ByRef tPixelFormat As D2D1_PIXEL_FORMAT, _
                Optional ByVal eType As D2D1_RENDER_TARGET_TYPE = D2D1_RENDER_TARGET_TYPE_DEFAULT, _
                Optional ByVal fDpiX As Single, _
                Optional ByVal fDpiY As Single, _
                Optional ByVal eUsage As D2D1_RENDER_TARGET_USAGE = D2D1_RENDER_TARGET_USAGE_NONE, _
                Optional ByVal eMinLevel As D2D1_FEATURE_LEVEL = _
                D2D1_FEATURE_LEVEL_DEFAULT) As D2D1_RENDER_TARGET_PROPERTIES
    
    With RenderTargetProperties
    
    .type = eType
    .pixelFormat = tPixelFormat
    .DpiX = fDpiX
    .DpiY = fDpiY
    .usage = eUsage
    .minLevel = eMinLevel
    
    End With
    
End Function

Public Function HwndRenderTargetProperties( _
                ByVal hWnd As Long, _
                ByRef tPixelSize As D2D1_SIZE_U, _
                Optional ByVal ePresentOptions As D2D1_PRESENT_OPTIONS = _
                D2D1_PRESENT_OPTIONS_NONE) As D2D1_HWND_RENDER_TARGET_PROPERTIES
    
    With HwndRenderTargetProperties
    
    .hWnd = hWnd
    .pixelSize = tPixelSize
    .presentOptions = ePresentOptions
    
    End With
    
End Function

Public Function LayerParameters( _
                ByRef tContentBounds As D2D1_RECT_F, _
                ByVal cGeometricMask As ID2D1Geometry, _
                ByVal eMaskAntialiasMode As D2D1_ANTIALIAS_MODE, _
                ByRef tMaskTransform As D2D1_MATRIX_3X2_F, _
                Optional ByVal fOpacity As Single = 1!, _
                Optional ByVal cOpacityBrush As ID2D1Brush, _
                Optional ByVal eLayerOptions As D2D1_LAYER_OPTIONS = D2D1_LAYER_OPTIONS_NONE) As D2D1_LAYER_PARAMETERS
    
    With LayerParameters
    
    .contentBounds = tContentBounds
    Set .geometricMask = cGeometricMask
    .maskAntialiasMode = eMaskAntialiasMode
    .maskTransform = tMaskTransform
    .opacity = fOpacity
    Set .opacityBrush = cOpacityBrush
    .layerOptions = eLayerOptions
    
    End With
    
End Function

Public Function DrawingStateDescription( _
                ByVal eAntialiasMode As D2D1_ANTIALIAS_MODE, _
                ByVal eTextAntialiasMode As D2D1_TEXT_ANTIALIAS_MODE, _
                ByVal cTag1 As Currency, _
                ByVal cTag2 As Currency, _
                ByRef tTransform As D2D1_MATRIX_3X2_F) As D2D1_DRAWING_STATE_DESCRIPTION
    
    With DrawingStateDescription
    
    .antialiasMode = eAntialiasMode
    .textAntialiasMode = eTextAntialiasMode
    .tag1 = cTag1
    .tag2 = cTag2
    .Transform = tTransform
    
    End With
    
End Function

Public Function ColorF( _
                ByVal eColor As D2D1_COLORS, _
                Optional ByVal fAlpha As Single = 1!) As D2D1_COLOR_F
    Dim bR As Byte
    Dim bB As Byte
    Dim bG As Byte
    
    bB = eColor And &HFF
    bG = (eColor \ &H100) And &HFF
    bR = (eColor \ &H10000) And &HFF
    
    With ColorF
    
    .r = bR / 255!
    .g = bG / 255!
    .b = bB / 255!
    .a = fAlpha
    
    End With
    
End Function


Public Function ColorF3( _
                ByVal eColor As Long, _
                Optional ByVal fAlpha As Single = 1!) As D2D1_COLOR_F
    Dim bR As Byte
    Dim bB As Byte
    Dim bG As Byte
    
    If (eColor And &H80000000) Then eColor = GetSysColor(eColor And &HFF&)
    
    bB = (eColor \ &H10000) And &HFF
    bG = (eColor \ &H100) And &HFF
    bR = eColor And &HFF
    
    With ColorF3
    
    .r = bR / 255!
    .g = bG / 255!
    .b = bB / 255!
    .a = fAlpha
    
    End With
    
End Function






Public Function ColorF2( _
                ByVal fR As Single, _
                ByVal fG As Single, _
                ByVal fB As Single, _
                Optional ByVal fAlpha As Single = 1!) As D2D1_COLOR_F
    
    With ColorF2
    
    .r = fR
    .g = fG
    .b = fB
    .a = fAlpha
    
    End With
    
End Function

'Public Function Matrix3x2F( _
'                ByVal f_11 As Single, _
'                ByVal f_12 As Single, _
'                ByVal f_21 As Single, _
'                ByVal f_22 As Single, _
'                ByVal f_31 As Single, _
'                ByVal f_32 As Single) As D2D1_MATRIX_3X2_F
'
'    With Matrix3x2F
'    .m_11 = f_11
'    .m_12 = f_12
'    .m_21 = f_21
'    .m_22 = f_22
'    .m_31 = f_31
'    .m_32 = f_32
'
'    End With
'
'End Function
'
Public Function Matrix3x2F_Identity() As D2D1_MATRIX_3X2_F

    With Matrix3x2F_Identity

    .M(0, 0) = 1
    .M(1, 1) = 1

    End With

End Function
'
'Public Function Matrix3x2F_Translation( _
'                ByRef tSize As D2D1_SIZE_F) As D2D1_MATRIX_3X2_F
'
'    With Matrix3x2F_Translation
'
'    .m_11 = 1!: .m_12 = 0!
'    .m_21 = 0!: .m_22 = 1!
'    .m_31 = tSize.width: .m_32 = tSize.height
'
'    End With
'
'End Function
'
'Public Function Matrix3x2F_Translation2( _
'                ByVal fWidth As Single, _
'                ByVal fHeight As Single) As D2D1_MATRIX_3X2_F
'
'    With Matrix3x2F_Translation2
'
'    .m_11 = 1!: .m_12 = 0!
'    .m_21 = 0!: .m_22 = 1!
'    .m_31 = fWidth: .m_32 = fHeight
'
'    End With
'
'End Function
'
'Public Function Matrix3x2F_Scale( _
'                ByRef tSize As D2D1_SIZE_F, _
'                ByRef tCenter As D2D1_POINT_2F) As D2D1_MATRIX_3X2_F
'
'    With Matrix3x2F_Scale
'
'    .m_11 = tSize.width: .m_12 = 0!
'    .m_21 = 0!: .m_22 = tSize.height
'    .m_31 = tCenter.x - tSize.width * tCenter.x
'    .m_32 = tCenter.y - tSize.height * tCenter.y
'
'    End With
'
'End Function
'
'Public Function Matrix3x2F_Scale2( _
'                ByVal fWidth As Single, _
'                ByVal fHeight As Single, _
'                ByRef tCenter As D2D1_POINT_2F) As D2D1_MATRIX_3X2_F
'
'    With Matrix3x2F_Scale2
'
'    .m_11 = fWidth: .m_12 = 0!
'    .m_21 = 0!: .m_22 = fHeight
'    .m_31 = tCenter.x - fWidth * tCenter.x
'    .m_32 = tCenter.y - fHeight * tCenter.y
'
'    End With
'
'End Function
'
'Public Function Matrix3x2F_Rotation( _
'                ByVal fAngle As Single, _
'                ByRef tCenter As D2D1_POINT_2F) As D2D1_MATRIX_3X2_F
'    D2D1MakeRotateMatrix fAngle, tCenter.x, tCenter.y, Matrix3x2F_Rotation
'End Function
'
'Public Function Matrix3x2F_Rotation2( _
'                ByVal fAngle As Single, _
'                ByVal fCenterX As Single, _
'                ByVal fCentery As Single) As D2D1_MATRIX_3X2_F
'    D2D1MakeRotateMatrix fAngle, fCenterX, fCentery, Matrix3x2F_Rotation2
'End Function
'
'Public Function Matrix3x2F_Skew( _
'                ByVal fAngleX As Single, _
'                ByVal fAngleY As Single, _
'                ByRef tCenter As D2D1_POINT_2F) As D2D1_MATRIX_3X2_F
'    D2D1MakeSkewMatrix fAngleX, fAngleY, tCenter.x, tCenter.y, Matrix3x2F_Skew
'End Function
'
'Public Function Matrix3x2F_Determinant( _
'                ByRef tMtx As D2D1_MATRIX_3X2_F) As Single
'    Matrix3x2F_Determinant = (tMtx.m_11 * tMtx.m_22) - (tMtx.m_12 * tMtx.m_21)
'End Function
'
'Public Function Matrix3x2F_IsInvertible( _
'                ByRef tMtx As D2D1_MATRIX_3X2_F) As Boolean
'    Matrix3x2F_IsInvertible = D2D1IsMatrixInvertible(tMtx)
'End Function
'
'Public Function Matrix3x2F_Invert( _
'                ByRef tMtx As D2D1_MATRIX_3X2_F) As Boolean
'    Matrix3x2F_Invert = D2D1InvertMatrix(tMtx)
'End Function
'
'Public Function Matrix3x2F_IsIdentity( _
'                ByRef tMtx As D2D1_MATRIX_3X2_F) As Boolean
'    Matrix3x2F_IsIdentity = (tMtx.m_11 = 1!) And (tMtx.m_12 = 0!) And _
'                            (tMtx.m_21 = 0!) And (tMtx.m_22 = 1!) And _
'                            (tMtx.m_31 = 0!) And (tMtx.m_32 = 0!)
'End Function
'
'Public Function Matrix3x2F_SetProduct( _
'                ByRef tMtx1 As D2D1_MATRIX_3X2_F, _
'                ByRef tMtx2 As D2D1_MATRIX_3X2_F) As D2D1_MATRIX_3X2_F
'
'    With Matrix3x2F_SetProduct
'
'    .m_11 = tMtx1.m_11 * tMtx2.m_11 + tMtx1.m_12 * tMtx2.m_21
'    .m_12 = tMtx1.m_11 * tMtx2.m_12 + tMtx1.m_12 * tMtx2.m_22
'    .m_21 = tMtx1.m_21 * tMtx2.m_11 + tMtx1.m_22 * tMtx2.m_21
'    .m_22 = tMtx1.m_21 * tMtx2.m_12 + tMtx1.m_22 * tMtx2.m_22
'    .m_31 = tMtx1.m_31 * tMtx2.m_11 + tMtx1.m_32 * tMtx2.m_21 + tMtx2.m_31
'    .m_32 = tMtx1.m_31 * tMtx2.m_12 + tMtx1.m_32 * tMtx2.m_22 + tMtx2.m_32
'
'    End With
'
'End Function
'
'Public Function Matrix3x2F_TransformPoint( _
'                ByRef tMtx As D2D1_MATRIX_3X2_F, _
'                ByRef tPoint As D2D1_POINT_2F) As D2D1_POINT_2F
'
'    With Matrix3x2F_TransformPoint
'
'    .x = tPoint.x * tMtx.m_11 + tPoint.y * tMtx.m_21 + tMtx.m_31
'    .y = tPoint.x * tMtx.m_12 + tPoint.y * tMtx.m_22 + tMtx.m_32
'
'    End With
'
'End Function

Public Function UUID_NULL() As UUID
Static bSet As Boolean
Static IID As UUID
If bSet = False Then
  With IID
    .Data1 = 0: .Data2 = 0: .Data3 = 0
    .Data4(0) = 0: .Data4(1) = 0: .Data4(2) = 0: .Data4(3) = 0: .Data4(4) = 0: .Data4(5) = 0: .Data4(6) = 0: .Data4(7) = 0
  End With
End If
bSet = True
UUID_NULL = IID
End Function
